@@ -3,12 +3,14 @@ use anyhow::{anyhow, Result};
 use chrono::prelude::*;
 use clap::Parser;
 use crossterm::event::KeyModifiers;
+use crossterm::terminal::{EnterAlternateScreen, LeaveAlternateScreen};
 use crossterm::{
     event::{self, Event as CEvent, KeyCode},
     execute,
     terminal::{disable_raw_mode, enable_raw_mode, SetSize},
 };
 use dns_lookup::lookup_host;
+use itertools::{Itertools, MinMaxResult};
 use pinger::{ping_with_interval, PingResult};
 use std::io;
 use std::io::BufWriter;
@@ -23,7 +25,7 @@ use std::thread;
 use std::thread::{sleep, JoinHandle};
 use std::time::{Duration, Instant};
 use tui::backend::{Backend, CrosstermBackend};
-use tui::layout::{Constraint, Direction, Layout};
+use tui::layout::{Constraint, Direction, Flex, Layout};
 use tui::style::{Color, Style};
 use tui::text::Span;
 use tui::widgets::{Axis, Block, Borders, Chart, Dataset};
@@ -35,6 +37,7 @@ mod region_map;
 
 use colors::Colors;
 use shadow_rs::{formatcp, shadow};
+use tui::prelude::Position;
 
 shadow!(build);
 
@@ -104,17 +107,27 @@ struct Args {
         long = "color",
         use_value_delimiter = true,
         value_delimiter = ',',
-        help = "\
-            Assign color to a graph entry. This option can be defined more than \
-            once as a comma separated string, and the order which the colors are \
-            provided will be matched against the hosts or commands passed to gping. \
-            Hexadecimal RGB color codes are accepted in the form of '#RRGGBB' or the \
-            following color names: 'black', 'red', 'green', 'yellow', 'blue', 'magenta',\
-            'cyan', 'gray', 'dark-gray', 'light-red', 'light-green', 'light-yellow', \
-            'light-blue', 'light-magenta', 'light-cyan', and 'white'\
-        "
+        help = r#"Assign color to a graph entry. 
+
+This option can be defined more than once as a comma separated string, and the 
+order which the colors are provided will be matched against the hosts or 
+commands passed to gping.
+
+Hexadecimal RGB color codes are accepted in the form of '#RRGGBB' or the 
+following color names: 'black', 'red', 'green', 'yellow', 'blue', 'magenta',
+'cyan', 'gray', 'dark-gray', 'light-red', 'light-green', 'light-yellow', 
+'light-blue', 'light-magenta', 'light-cyan', and 'white'"#
     )]
     color_codes_or_names: Vec<String>,
+    #[arg(
+        name = "clear",
+        long = "clear",
+        help = "\
+            Clear the graph from the terminal after closing the program\
+        ",
+        action
+    )]
+    clear: bool,
 }
 
 struct App {
@@ -141,13 +154,19 @@ impl App {
         // Find the Y axis bounds for our chart.
         // This is trickier than the x-axis. We iterate through all our PlotData structs
         // and find the min/max of all the values. Then we add a 10% buffer to them.
-        let iter = self
+        let (min, max) = match self
             .data
             .iter()
             .flat_map(|b| b.data.as_slice())
-            .map(|v| v.1);
-        let min = iter.clone().fold(f64::INFINITY, |a, b| a.min(b));
-        let max = iter.fold(0f64, |a, b| a.max(b));
+            .map(|v| v.1)
+            .filter(|v| !v.is_nan())
+            .minmax()
+        {
+            MinMaxResult::NoElements => (f64::INFINITY, 0_f64),
+            MinMaxResult::OneElement(elm) => (elm, elm),
+            MinMaxResult::MinMax(min, max) => (min, max),
+        };
+
         // Add a 10% buffer to the top and bottom
         let max_10_percent = (max * 10_f64) / 100_f64;
         let min_10_percent = (min * 10_f64) / 100_f64;
@@ -171,12 +190,12 @@ impl App {
     }
 
     fn x_axis_labels(&self, bounds: [f64; 2]) -> Vec<Span> {
-        let lower_utc = NaiveDateTime::from_timestamp_opt(bounds[0] as i64, 0)
+        let lower_utc = DateTime::<Utc>::from_timestamp(bounds[0] as i64, 0)
             .expect("Error parsing x-axis bounds 0");
-        let upper_utc = NaiveDateTime::from_timestamp_opt(bounds[1] as i64, 0)
+        let upper_utc = DateTime::<Utc>::from_timestamp(bounds[1] as i64, 0)
             .expect("Error parsing x-asis bounds 1");
-        let lower = Local::from_utc_datetime(&Local, &lower_utc);
-        let upper = Local::from_utc_datetime(&Local, &upper_utc);
+        let lower: DateTime<Local> = DateTime::from(lower_utc);
+        let upper: DateTime<Local> = DateTime::from(upper_utc);
         let diff = (upper - lower) / 2;
         let midpoint = lower + diff;
         vec![
@@ -406,7 +425,16 @@ fn main() -> Result<()> {
     let stdout = io::stdout();
     let mut backend = CrosstermBackend::new(BufWriter::with_capacity(1024 * 1024 * 4, stdout));
     let rect = backend.size()?;
-    execute!(backend, SetSize(rect.width, rect.height),)?;
+
+    if args.clear {
+        execute!(
+            backend,
+            SetSize(rect.width, rect.height),
+            EnterAlternateScreen,
+        )?;
+    } else {
+        execute!(backend, SetSize(rect.width, rect.height),)?;
+    }
 
     let mut terminal = Terminal::new(backend)?;
     terminal.clear()?;
@@ -453,6 +481,7 @@ fn main() -> Result<()> {
             Event::Render => {
                 terminal.draw(|f| {
                     let chunks = Layout::default()
+                        .flex(Flex::Legacy)
                         .direction(Direction::Vertical)
                         .vertical_margin(args.vertical_margin)
                         .horizontal_margin(args.horizontal_margin)
@@ -460,10 +489,9 @@ fn main() -> Result<()> {
                             iter::repeat(Constraint::Length(1))
                                 .take(app.data.len())
                                 .chain(iter::once(Constraint::Percentage(10)))
-                                .collect::<Vec<_>>()
-                                .as_ref(),
+                                .collect::<Vec<_>>(),
                         )
-                        .split(f.size());
+                        .split(f.area());
 
                     let total_chunks = chunks.len();
 
@@ -488,8 +516,7 @@ fn main() -> Result<()> {
                             )
                             .split(*chunk);
 
-                        for (area, paragraph) in
-                            header_layout.into_iter().zip(plot_data.header_stats())
+                        for (area, paragraph) in header_layout.iter().zip(plot_data.header_stats())
                         {
                             f.render_widget(paragraph, *area);
                         }
@@ -531,10 +558,17 @@ fn main() -> Result<()> {
     terminal.show_cursor()?;
 
     let new_size = terminal.size()?;
-    terminal.set_cursor(new_size.width, new_size.height)?;
+    terminal.set_cursor_position(Position {
+        x: new_size.width,
+        y: new_size.height,
+    })?;
     for thread in threads {
         thread.join().unwrap()?;
     }
 
+    if args.clear {
+        execute!(terminal.backend_mut(), LeaveAlternateScreen)?;
+    };
+
     Ok(())
 }
